from typing import Set, Tuple, Dict, Union
from common import Terminal, Nonterminal, State, Character, Eps, Emptyset
from grammars import RegGrammar, CFG
from dfa import DFA
from nfa import NFA
from regex import RegEx
from parser import Parser, ParsingError


class RegGrammar(RegGrammar):
    pass

class Grammar(CFG):
    pass


class DFA(DFA):
    def dfa_to_nfa(self) -> NFA:
        new_transition: NFA.transition = {}
        for (state, character) in self.transition:
            new_transition[state, character] = {self.transition[state, character]}
        return NFA(self.states, self.characters, new_transition, self.init, self.final)


class NFA(NFA):
    def nfa_to_reggrammar(self) -> RegGrammar:
        nonterminals: Set[Nonterminal] = set()
        terminals: Set[Terminal] = set()
        rules: RegGrammar.Rules = dict()
        init = Nonterminal(self.init.name)

        for state in self.states:
            nonterminals.add(Nonterminal(state.name))
        for character in self.characters:
            terminals.add(Terminal(character.name))

        for state, character in self.transition:
            for dest_state in self.transition[state, character]:
                nonterminal1 = Nonterminal(state.name)
                nonterminal2 = Nonterminal(dest_state.name)
                terminal = Terminal(character.name)
                rules.setdefault(nonterminal1, set()).add((terminal, nonterminal2))

                if dest_state in self.final:
                    rules[nonterminal1].add(terminal)

        if self.init in self.final:
            new_init = Nonterminal("new_init")
            rules[new_init] = {Eps}
            for rule in rules[init]:
                rules[new_init].add(rule)
            init = new_init

        return RegGrammar(nonterminals, terminals, rules, init)

    def nfa_to_regex(self) -> RegEx:
        # I don't want to have to do this really don't
        pass

class RegEx(RegEx):
    pass


class Parser(Parser):
    pass


class ParsingError(ParsingError):
    pass
